import uuid
from django.db import models
from django.contrib.auth.models import User
from datetime import datetime
from . import utils


class LocationData(models.Model):
    location_id = models.CharField(max_length=32, blank=False, null=False, name="Standort-ID")
    table_no = models.IntegerField(blank=False, null=False, name="Tischnummer")
    tst = models.DateTimeField(blank=False, null=False, name="Zeitstempel")
    on_off = models.BooleanField(blank=False, null=False, name="Ein/Ausgebucht")
    processed = models.BooleanField(default=False, name="Verarbeitet")
    error_msg = models.CharField(max_length=16000, blank=True, null=True, name="Fehlermeldung")

    def __str__(self):
        return str(self.location_id)

    class Meta:
        verbose_name = "Standortlog"
        verbose_name_plural = "Standortlogs"

class Location(models.Model):
    users = models.ManyToManyField(User, related_name='locations', name="Benutzer")
    code = models.CharField(max_length=16, unique=True, name="Code")
    name = models.CharField(max_length=64, unique=True, name="Name")
    street = models.CharField(max_length=64, blank=True, null=True, name="Straße")
    plz = models.CharField(max_length=8, blank=True, null=True, name="Postleitzahl")
    city = models.CharField(max_length=64, blank=True, null=True, name="Stadt")
    phone = models.CharField(max_length=64, blank=True, null=True, name="Telefon")
    email = models.EmailField(blank=True, null=True, name="Email")
    url = models.URLField(blank=True,null=True, name="URL")

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = "Standort"
        verbose_name_plural = "Standorte"


class Client(models.Model):
    uuid = models.UUIDField(unique=True, default=uuid.uuid4, name="Identifier")
    location = models.ForeignKey(Location, name="Standort")
    desk1_enable = models.BooleanField()
    desk1_name = models.CharField(max_length=32, blank=True, null=True)
    desk1_prize_nt = models.DecimalField(max_digits=4, decimal_places=2, blank=True, null=True)
    desk1_prize_ht = models.DecimalField(max_digits=4, decimal_places=2, blank=True, null=True)
    desk2_enable = models.BooleanField()
    desk2_name = models.CharField(max_length=32, blank=True, null=True)
    desk2_prize_nt = models.DecimalField(max_digits=4, decimal_places=2, blank=True, null=True)
    desk2_prize_ht = models.DecimalField(max_digits=4, decimal_places=2, blank=True, null=True)

    def accounting_for(self, desk_no, pht, pnt):
        t = Accounting.objects.filter(client_id=self.id, desk_no=desk_no)[:3][::-1]
        a = t[t.__len__() - 1]
        if a.time_to is None:
            prize = utils.get_prize_for(start=a.time_from, end=datetime.now(), pph=pht)
            if prize != a.prize:
                a.prize = prize
        return t

    def accounting_1(self):
        return self.accounting_for(1, self.desk1_prize_ht, self.desk1_prize_nt)

    def accounting_2(self):
        return self.accounting_for(2, self.desk2_prize_ht, self.desk2_prize_nt)

    def __str__(self):
        return '{}, {}'.format(self.location.name, self.uuid)

    class Meta:
        verbose_name = "Client"
        verbose_name_plural = "Clienten"

class Accounting(models.Model):
    client = models.ForeignKey(Client, name="Client")
    desk_no = models.IntegerField(name="Tischnummer")
    time_from = models.DateTimeField(name="Beginn")
    time_to = models.DateTimeField(blank=True, null=True, name="Ende")
    prize = models.DecimalField(max_digits=5, decimal_places=2, blank=True, null=True, name="Preis")

    def __str__(self):
        return '{}: {} -> {}, {}'.format(self.client.uuid, self.time_from, self.time_to, self.prize)

    class Meta:
        ordering = ['-time_from']
        verbose_name = "Buchhaltungseintrag"
        verbose_name_plural = "Buchhaltungseinträge"